import asyncio
import logging
import sys
import time

from autobahn.asyncio.wamp import ApplicationRunner, ApplicationSession
from autobahn.wamp.types import ComponentConfig
from pydantic import validate_call
from .backends.common import Response
from mev_common.models import LoginData, PlateDb, QueryStatus

log = logging.getLogger(__name__)


class Interface(ApplicationSession):
    # api: api_cloud.Api  # valorizzato dal login al cloud
    # api: Api

    def __init__(self, config):
        ApplicationSession.__init__(self, ComponentConfig(config.realm))
        self.settings = config
        self.latency: float = 0  # latest latency of query

        if config.backend == "fake-s21":
            from .backends.s21 import FakeS21

            self.api = FakeS21(
                config=config,
                backend=config.backend,
                forced_url=config.forced_url,
            )
        elif config.backend == "s21":
            from .backends.s21 import S21

            self.api = S21(config=config, backend=config.backend)
        else:
            log.error("backend not configured %s", config.backend)
            sys.exit(1)

        runner = ApplicationRunner(config.ws, config.realm, extra={})
        asyncio.get_event_loop().run_until_complete(runner.run(self, start_loop=False))

    async def onJoin(self, details):
        log.info("wamp session attached to realm %s", self.settings.realm)
        self.subscribe(self.on_plate, "plate.valid")
        self.register(self.setlogin, "query.setlogin")
        self.register(self.setbackend, "query.setbackend")

        asyncio.create_task(self.status())
        while self.settings.test_query:
            await self.on_plate(plate={"plate": "EN675TL", "nationality": "ITA"})
            await asyncio.sleep(3)

    async def status(self):
        """check status of remote service"""
        while True:
            if self.api.autologin and not self.api.login:
                await self.api.get_token()
            self.publish(
                "query.status",
                status=QueryStatus(
                    logged=self.api.login, latency=self.latency, up=True
                ).dict(),
            )
            await asyncio.sleep(3)

    @validate_call
    async def setlogin(self, login_data: LoginData):
        return await self.api.set_login(login_data)

    async def setbackend(self, backend: str):
        self.api.set_backend(backend)
        return await self.api.get_token()

    def onDisconnect(self):
        log.warning("wamp transport lost, shutting down")
        asyncio.get_event_loop().stop()

    @validate_call
    async def on_plate(self, plate: PlateDb):
        """
        arrivano le targhe valide dal brain o dal frontend (per riprovare una query fallita)
        """

        if plate.nationality != "ITA":
            # plate_query.query = {"error": "foreign"}
            log.info("query skip plate=%s nationality not ITA", plate.plate)
        else:
            log.info("query plate=%s", plate)
            if not self.api.login:
                await self.api.get_token()
            if self.api.login:
                start_time = time.time()
                try:
                    response: Response | None = await self.api.base_query(plate)
                    if response:
                        response.query.latency = int(1000 * (time.time() - start_time))
                        log.info("query result: %s", response)
                        self.publish(
                            "plate.query",
                            plate=response.dict(),
                        )
                    else:
                        log.warning("query with errors")
                except Exception as e:
                    log.error("backend %s error: %s", self.api.backend, e)

    def close(self):
        pass


def main():
    from .config import get_config

    config = get_config()

    interface = Interface(config)
    try:
        asyncio.get_event_loop().run_forever()
    except KeyboardInterrupt:
        interface.close()
    finally:
        print("exit from mev_query")
